﻿using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace SLMultiThread.ThreadTools
{
    // (c) Copyright Microsoft Corporation.
    // This source is subject to the Microsoft Public License (Ms-PL).
    // Please see http://go.microsoft.com/fwlink/?LinkID=131993 for details.
    // All other rights reserved.

        /// <summary>
        /// A base class for data objects that implement the property changed
        /// interface, offering data binding and change notifications.
        /// </summary>
        public class PropertyChangedBase : INotifyPropertyChanged
        {
            /// <summary>
            /// A static set of argument instances, one per property name.
            /// </summary>
            private static readonly Dictionary<string, PropertyChangedEventArgs> argumentInstances = 
                new Dictionary<string, PropertyChangedEventArgs>();

            /// <summary>
            /// The property changed event.
            /// </summary>
            public event PropertyChangedEventHandler PropertyChanged;

            /// <summary>
            /// Notify any listeners that the property value has changed.
            /// </summary>
            /// <param name="propertyName">The property name.</param>
            protected void NotifyPropertyChanged(string propertyName)
            {
                if (string.IsNullOrEmpty(propertyName))
                {
                    throw new ArgumentException("PropertyName cannot be empty or null.");
                }

                var handler = PropertyChanged;
                if (handler == null) return;
                PropertyChangedEventArgs args;
                if (!argumentInstances.TryGetValue(propertyName, out args))
                {
                    args = new PropertyChangedEventArgs(propertyName);
                    argumentInstances[propertyName] = args;
                }

                // Fire the change event. The smart dispatcher will directly
                // invoke the handler if this change happened on the UI thread,
                // otherwise it is sent to the proper dispatcher.
                SmartDispatcher.BeginInvoke(() => handler(this, args));
            }
        }

}
